//
//  KBGradientPopUpButton.m
//  -----------------------
//
//  Created by Keith Blount on 19/03/2006.
//  

#import "KBGradientPopUpButton.h"


@interface KBGradientPopUpButtonCell : NSPopUpButtonCell
{
	BOOL usesFullScreenStyle;
}
- (void)setUsesFullScreenStyle:(BOOL)flag;
- (BOOL)usesFullScreenStyle;
@end

@implementation KBGradientPopUpButtonCell

- (id)initTextCell:(NSString *)stringValue pullsDown:(BOOL)pullDown
{
	if (self = [super initTextCell:stringValue pullsDown:pullDown])
	{
		usesFullScreenStyle = NO;
		[self setControlSize:NSSmallControlSize];
	}
	return self;
}

- (void)setUsesFullScreenStyle:(BOOL)flag
{
	usesFullScreenStyle = flag;
}

- (BOOL)usesFullScreenStyle
{
	return usesFullScreenStyle;
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	// Note full screen style does not currently support a disabled state...
	NSString *state = [self isEnabled] ? ([self isHighlighted] ? @"P" : @"N") : (usesFullScreenStyle ? @"N" : @"D");
	NSString *leftBaseName = usesFullScreenStyle ? @"fs-popupLeft-%@" : @"tools-popupLeft-%@";
	NSString *fillBaseName = usesFullScreenStyle ? @"fs-popupFill-%@" : @"tools-popupFill-%@";
	NSString *rightBaseName = usesFullScreenStyle ? @"fs-popupRight-%@" : @"tools-popupRight-%@";
	NSImage *leftImg = [NSImage imageNamed:[NSString stringWithFormat:leftBaseName, state]];
	NSImage *fillImg = [NSImage imageNamed:[NSString stringWithFormat:fillBaseName, state]];
	NSImage *rightImg = [NSImage imageNamed:[NSString stringWithFormat:rightBaseName, state]];
	
	NSSize s = [leftImg size];
	float y = NSMaxY(cellFrame)-((cellFrame.size.height-s.height)/2.0)-1.0;
	
	float fillX = cellFrame.origin.x + 2.0 + s.width;
	float fillWidth = cellFrame.size.width - s.width;
	
	NSPoint p = NSMakePoint(cellFrame.origin.x+2.0, y);
	[leftImg compositeToPoint:p
					operation:NSCompositeSourceOver];
	
	s = [rightImg size];
	p = NSMakePoint(NSMaxX(cellFrame)-s.width-2.0, y);
	[rightImg compositeToPoint:p
					 operation:NSCompositeSourceOver];
	
	fillWidth -= s.width+4.0;
	
	p = NSMakePoint(fillX, y);
	[fillImg setScalesWhenResized:YES];
	[fillImg setSize:NSMakeSize(fillWidth,[fillImg size].height)];
	[fillImg compositeToPoint:p
				operation:NSCompositeSourceOver];
	
	[self drawInteriorWithFrame:cellFrame inView:controlView];
}

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
	if (!usesFullScreenStyle)
	{
		[super drawInteriorWithFrame:cellFrame inView:controlView];
		return;
	}
	
	NSMutableParagraphStyle *style = [[[NSMutableParagraphStyle alloc] init] autorelease];
	[style setLineBreakMode:NSLineBreakByTruncatingTail];
	NSAttributedString *attrTitle = [[[NSAttributedString alloc] initWithString:[self title]
															  attributes:[NSDictionary dictionaryWithObjectsAndKeys:
																  [NSColor whiteColor], NSForegroundColorAttributeName,
																  [NSFont systemFontOfSize:10.0], NSFontAttributeName,
																  style, NSParagraphStyleAttributeName,
																  nil]] autorelease];
	NSSize s = [attrTitle size];
	float leftOffset = 12.0;
	float rightOffset = [[NSImage imageNamed:@"fs-popupRight-N"] size].width;
	NSRect titleRect = NSMakeRect(leftOffset,
								  3.0,
								  cellFrame.size.width-leftOffset-rightOffset,
								  s.height);
	[attrTitle drawInRect:titleRect];
}

@end


@implementation KBGradientPopUpButton

- (id)initWithCoder:(NSCoder *)decoder
{
	if ( self = [super initWithCoder:decoder] )
	{
		if (![[self cell] isKindOfClass:[KBGradientPopUpButtonCell class]])
		{
			NSMenu *theMenu = [[self menu] copyWithZone:[self zone]];
			[self setCell:[[[KBGradientPopUpButtonCell alloc] initTextCell:[self title] 
																 pullsDown:[self pullsDown]] autorelease]];
			[self setMenu:theMenu];
			[theMenu release];
		}
		
		[self setFont:[NSFont systemFontOfSize:11.0]];
		
		[self setUsesFullScreenStyle:NO];
	}
	return self;
}

+ (Class)cellClass
{
	return [KBGradientPopUpButtonCell class];
}

- (void)setUsesFullScreenStyle:(BOOL)flag
{
	[(KBGradientPopUpButtonCell *)[self cell] setUsesFullScreenStyle:flag];
	[self setNeedsDisplay:YES];
}

- (BOOL)usesFullScreenStyle
{
	return [(KBGradientPopUpButtonCell *)[self cell] usesFullScreenStyle];
}

@end
